@@ -550,6 +550,24 @@ bool CommandLine::Parse(int argc, char *argv[])
         }
         else
         {
+          //start of shell expanded * patch. -- Michael Evans
+          //The shell might expaned * so, if we have our name and we're creating, then filter for files...
+          if ((parfilename.length() != 0) && (operation == opCreate))
+          {
+#ifdef WIN32
+            if (GetFileAttributes(argv[0]) & FILE_ATTRIBUTE_DIRECTORY) // != 0, but no need...
+#else //Not WIN32, probably *nix
+            struct stat st;
+            if (!(stat(argv[0], &st) == 0 && S_ISREG(st.st_mode)))
+#endif
+            {
+              cerr << "Skipping non-regular file: " << argv[0] << endl;
+              argc--;
+              argv++;
+              options = false;
+              continue;
+            }
+          }//end of shell expanded * patch. -- Michael Evans
           filenames = new list<string>;
           filenames->push_back(argv[0]);
         }
