@@ -893,7 +893,7 @@ bool DiskFile::Rename(void)
   do
   {
     int length = snprintf(newname, _MAX_PATH, "%s.%d", filename.c_str(), ++index);
-    if (length < 0)
+    if (length < 0 || length >= _MAX_PATH)
     {
       cerr << filename << " cannot be renamed." << endl;
       return false;
